package MyNewAgent;

import YOLOBOT.YoloState;
import YOLOBOT.Util.Wissensdatenbank.PlayerEvent;
import YOLOBOT.Util.Wissensdatenbank.YoloEvent;
import YOLOBOT.Util.Wissensdatenbank.YoloKnowledge;
import core.game.Observation;
import ontology.Types.ACTIONS;

public class KnowledgeBaseExamples {

	
	/**
	 * Predicts whether its stupid to executing an specific action given a state
	 * @param state game state to check the action
	 * @param action the action to check
	 * @param dyingIsStupid if true, method returns true if agent dies
	 * @param ignoreStochasticMovingEnemies if true, stochastic enemies dont move, else stochastic enemies movement is simulated as worst-case for agent.  
	 * @return true, iff move is stupid (dying, or logically equals null action)
	 */
	public boolean predictStupidMovement(YoloState state, ACTIONS action, boolean dyingIsStupid, boolean ignoreStochasticMovingEnemies ){
		
		return YoloKnowledge.instance.moveWillCancel(state, action, dyingIsStupid, ignoreStochasticMovingEnemies);
		
	}
	
	
	/**
	 * Predicts the outcome of a collision with agent and observation 
	 * @param state the current state (Agent information like itype and inventory needed)
	 * @param obs the Observation to predict agents collision with
	 * @return a YoloEvent predicting the collision or null if move will cancel
	 */
	public YoloEvent predictWhatHappensOnCollide(YoloState state, Observation obs){
		int avatarItype = state.getAvatar().itype;
		int observationItype = obs.itype;
		
		PlayerEvent event = YoloKnowledge.instance.getPlayerEvent(avatarItype, observationItype, true);
		int howOftenSeen = event.getObserveCount(); // How often has this collision been seen (higher values = more likely to predict correct)
		
		byte[] inventoryArray = state.getInventoryArray(); //Framework-Hashmap inventory transformed into byte array for further usage
		
		boolean willCancel = event.willCancel(inventoryArray);	//Is semantical equal to null move
		boolean isProbablyWall = (double)event.getCancelCount()/(double)event.getObserveCount() > 0.95;
		
		if(willCancel)
			return null;
		return event.getEvent(inventoryArray);
	}
	
	public void exampleYoloEventUsage(YoloEvent event){
		int inventorySlot = event.getAddInventorySlotItem();
		boolean winsState = event.getWinGame();
		boolean scoreIncrease = event.getScoreDelta() > 0;
		boolean inventoryIncrease = event.getAddInventorySlotItem() != -1;
		boolean decreaseScore = event.getScoreDelta() < 0;
		boolean killedOnColision = event.getKill();
		boolean iTypeChange = event.getIType() != -1;
	}
	
	
}
