package MyNewAgent.examples;

import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;

import YOLOBOT.YoloState;
import YOLOBOT.Util.Planner.KnowledgeBasedAStar;
import YOLOBOT.Util.Wissensdatenbank.PlayerEvent;
import YOLOBOT.Util.Wissensdatenbank.YoloEvent;
import YOLOBOT.Util.Wissensdatenbank.YoloKnowledge;
import core.game.Observation;
import ontology.Types;
import ontology.Types.ACTIONS;

public class TargetChooser {

	private HashMap<Integer, Integer> singleObjectCooldownSet;
	private LinkedList<InterestingTarged> possibleTargets;
	private int[] iTypeFoundCount;
	

	public TargetChooser() {
		singleObjectCooldownSet = new HashMap<Integer, Integer>();
	}


	private void chooseTarget(YoloState state) {
		KnowledgeBasedAStar kBA = new KnowledgeBasedAStar(state);
		kBA.setCountFoundItypes(true);
		kBA.setBlacklistedObjects(singleObjectCooldownSet.keySet());
		kBA.calculate(state.getAvatarX(), state.getAvatarY(),
				state.getAvatar().itype, new int[0], false, true, false);
		kBA.setBlacklistedObjects(null);
		
		Observation[] nearestITypesFound = kBA.nearestITypeObservationFound;
		iTypeFoundCount = kBA.iTypesFoundCount;
		possibleTargets = new LinkedList<InterestingTarged>();
		for (int i = 0; i < nearestITypesFound.length; i++) {
			if (nearestITypesFound[i] == null
					|| nearestITypesFound[i].category == Types.TYPE_FROMAVATAR
					|| nearestITypesFound[i].category == Types.TYPE_AVATAR) {
				continue;
			}

			InterestingTarged target = new InterestingTarged(
					nearestITypesFound[i]);
			calculatePriorityValue(target, iTypeFoundCount[i], state, kBA);
			possibleTargets.add(target);
		}
		InterestingTarged bestTarged = null;
		

		if (possibleTargets.isEmpty()) {
			//No target found at all!
			return;
		}
		
		//do best First:
		Collections.sort(possibleTargets);
		bestTarged = possibleTargets.getFirst();
		

		
		if(bestTarged.getPriorityValue() < -300) {
			//No good target found!
			return;
		}
		
			
		int bestX;
		int bestY;
		bestX = (int) (bestTarged.getObs().position.x / state.getBlockSize());
		bestY = (int) (bestTarged.getObs().position.y / state.getBlockSize());

		bestX = Math.max(0, Math.min(bestX, kBA.distance.length));
		bestY = Math.max(0, Math.min(bestY, kBA.distance[0].length));
		int distance = kBA.distance[bestX][bestY];	
		
		//Calculate from Target to avatar:
		int bestTargedIndex = YoloKnowledge.instance.itypeToIndex(bestTarged.getObs().itype);
		int targetFirstStopX = kBA.nearestITypeObservationFoundFirstTargetX[bestTargedIndex];
		int targetFirstStopY = kBA.nearestITypeObservationFoundFirstTargetY[bestTargedIndex];
		kBA.setMoveDirectionInverse(true);
		kBA.calculate(targetFirstStopX, targetFirstStopY, state.getAvatar().itype, new int[0], false, true, false);
		//Now kBA has a map with distances to target and shortest paths to target!

		//TODO: use kBA
		
		//Sample use: get actions to go to target
		int avatarPosX = state.getAvatarX();
		int avatarPosY = state.getAvatarY(); 
		LinkedList<ACTIONS> actionsToExecuteToGetToTarget =  kBA.extractActions(avatarPosX, avatarPosY);
		
		//Sample use: Calculate distance:
		int distanceToTarget = kBA.distance[avatarPosX][avatarPosY];
		
	}
	private double calculatePriorityValue(InterestingTarged target,
			int iTypeFoundCount, YoloState state, KnowledgeBasedAStar kBA) {
		Observation observation = target.getObs();
		double prioValue = 0;
		int posX = (int) (observation.position.x / state.getBlockSize());
		int posY = (int) (observation.position.y / state.getBlockSize());
		PlayerEvent pEvent = YoloKnowledge.instance.getPlayerEvent(
				state.getAvatar().itype, observation.itype, true);
		YoloEvent event = pEvent.getEvent(state.getInventoryArray());
		int slot = event.getAddInventorySlotItem();
		boolean winState = event.getWinGame();
		boolean scoreIncrease = event.getScoreDelta() > 0;
		boolean notSeenYet = pEvent.getObserveCount() == 0;
		boolean useActionEffective = YoloKnowledge.instance
				.canInteractWithUse(state.getAvatar().itype, observation.itype);
		boolean inventoryIncrease = event.getAddInventorySlotItem() != -1;
		boolean isBadSpawner = false;
		boolean decreaseScore = event.getScoreDelta() < 0;
		boolean willCancel = pEvent.willCancel(state.getInventoryArray());
		boolean isProbablyWall = (double)pEvent.getCancelCount()/(double)pEvent.getObserveCount() > 0.95;
		boolean killedOnColision = event.getKill();
		boolean isPortal = observation.category == Types.TYPE_PORTAL;
		boolean iTypeChange = event.getIType() != -1;
		boolean gotScoreThroughUseAction = YoloKnowledge.instance.getIncreaseScoreIfInteractWith(state.getAvatar().itype, observation.itype);
		
		//Is alone:
		int mask = state.getSimpleState().getMask(posX,posY);
		boolean standsOnWall = Integer.numberOfTrailingZeros(mask) + Integer.numberOfLeadingZeros(mask) != 31;

		if (YoloKnowledge.instance.isSpawner(observation.itype)) {
			isPortal = false;
			int iTypeIndexOfSpawner = YoloKnowledge.instance
					.getSpawnIndexOfSpawner(observation.itype);
			PlayerEvent spawnedPEvent = YoloKnowledge.instance.getPlayerEvent(
					state.getAvatar().itype,
					YoloKnowledge.instance.indexToItype(iTypeIndexOfSpawner),
					true);
			YoloEvent spawnedEvent = spawnedPEvent.getEvent(state
					.getInventoryArray());
			isBadSpawner = spawnedEvent.getKill() || spawnedPEvent.getObserveCount() == 0;
		}
		boolean isInventoryFull = false;
		if (inventoryIncrease) {

			isInventoryFull = YoloKnowledge.instance.getInventoryMax(slot) == state
					.getInventoryArray()[slot];
		}else{
			for (int i = 0; i < 5; i++) {
				if(YoloKnowledge.instance.getInventoryMax(i) == state.getInventoryArray()[i]){
					isInventoryFull = true;
					break;
				}
			}
			
		}
		double scoreIncreaseValue = event.getScoreDelta();
		int distanceToAvatar = kBA.distance[posX][posY];
		int index = YoloKnowledge.instance.itypeToIndex(observation.itype);
		
		//TODO: calculate priority values! (sample following)
		// Sample Dictionary-Lookup Priority values: 
		
		if (isBadSpawner && !useActionEffective) {
			prioValue -= 1000;
		}
		if (decreaseScore) {
			prioValue -= 1000;
		}
		if (distanceToAvatar <= 6) {
			prioValue += 21;
		}

		if (winState) {
			prioValue += 1000;
		} else if (notSeenYet && isPortal) {
			prioValue += 150;
		} else if (isPortal && isInventoryFull) {
			prioValue += 150;
		}else if (willCancel) {
			if(isProbablyWall){
				prioValue -= 250;
			}else
				prioValue -= 200;
		} else if (scoreIncrease && inventoryIncrease) {
			if(isInventoryFull)
				prioValue += 130;
			else
				prioValue += 140;
		} else if (scoreIncrease) {
			prioValue += 135;
		} else if (inventoryIncrease) {
			prioValue += 132;
		} else if (notSeenYet) {
			prioValue += 131;
		} else if (!killedOnColision && useActionEffective) {
			if(gotScoreThroughUseAction){
				prioValue += 130;
			}
			else
				prioValue += 90;
		} else if (iTypeChange) {
			prioValue += 100;
		} else if (killedOnColision && useActionEffective) {
			if(gotScoreThroughUseAction){
				if(YoloKnowledge.instance.getNpcMovesEveryXTicks(YoloKnowledge.instance.itypeToIndex(observation.itype))>2)
					prioValue += 125;
				else
					prioValue += 105;
			}
			else
				prioValue += 85;
		} else if (killedOnColision && !useActionEffective) {
			prioValue -= 100;
		}
		
		target.setPriorityValue(prioValue);
		target.setWinCondition(winState);
		target.setIsUseable(useActionEffective);
		target.setUnseen(notSeenYet);
		target.setDistance(distanceToAvatar);;
		target.setScoreIncrease(gotScoreThroughUseAction || scoreIncrease);
		return prioValue;
	}
}
